from is_core.forms.generic import BaseGenericInlineFormSet, smart_generic_inlineformset_factory


from .inline_form_views import InlineFormView


class GenericInlineFormView(InlineFormView):

    ct_fk_field = 'object_id'
    ct_field = 'content_type'

    base_inline_formset_class = BaseGenericInlineFormSet

    def get_formset_factory(self, fields=None, readonly_fields=()):
        return smart_generic_inlineformset_factory(
            self.model, self.request, form=self.get_form_class(), ct_field=self.ct_field, fk_field=self.ct_fk_field,
            extra=self.get_extra(), formset=self.base_inline_formset_class, exclude=self.get_exclude(), fields=fields,
            min_num=self.get_min_num(), max_num=self.get_max_num(), readonly_fields=readonly_fields,
            readonly=self.readonly, formreadonlyfield_callback=self.formfield_for_readonlyfield,
            formfield_callback=self.formfield_for_dbfield, labels=self._get_field_labels(),
            can_delete=self.get_can_delete()
        )


class TabularGenericInlineFormView(GenericInlineFormView):

    template_name = 'is_core/forms/tabular_inline_formset.html'


class StackedGenericInlineFormView(GenericInlineFormView):

    template_name = 'is_core/forms/stacked_inline_formset.html'


class ResponsiveGenericInlineFormView(GenericInlineFormView):

    template_name = 'is_core/forms/responsive_inline_formset.html'



from django.conf import settings
from django.conf.urls import url
from django.contrib.auth.decorators import login_required
from django.views.generic.base import RedirectView

from chamber.shortcuts import get_object_or_404

from is_core.exceptions import HTTPForbiddenResponseException
from is_core.site import get_model_core

from .models import ExportedFile


class DownloadExportedDataView(RedirectView):

    def get_redirect_url(self, *args, **kwargs):
        exported_file = get_object_or_404(
            ExportedFile.objects.filter_active(), slug=kwargs.get('slug')
        )
        core = get_model_core(ExportedFile)

        if (core.permission.has_permission('read_all', self.request, self, obj=exported_file)
                or (core.permission.has_permission('read_own', self.request, self, obj=exported_file)
                    and exported_file.created_by == self.request.user)):
            exported_file.downloaded_by.add(self.request.user)
            return exported_file.file.url
        else:
            raise HTTPForbiddenResponseException


def static_pyston():
    return [
        url(
            r'^{}/(?P<slug>.+)?/'.format(getattr(settings, 'PYSTON_DOWNLOAD_EXPORT_URL', 'export')),
            login_required(DownloadExportedDataView.as_view()),
            name='pyston-download-export'
        )
    ]



from germanium.decorators import login
from germanium.test_cases.client import ClientTestCase
from germanium.tools import assert_in

from .test_case import HelperTestCase, AsSuperuserTestCase


__all__ =(
    'UIOrderingTestCase',
)


class UIOrderingTestCase(AsSuperuserTestCase, HelperTestCase, ClientTestCase):
    ISSUE_UI_URL = '/issue/'

    @login(is_superuser=True)
    def test_superuser_may_read_users_grid(self):
        resp = self.get(self.ISSUE_UI_URL)
        assert_in(b'data-col="watched_by_string"', resp.content)



class FieldPermissionViewMixin:

    field_permissions = None

    def _get_field_permissions(self):
        return self.field_permissions if self.field_permissions is not None else self.core.field_permissions

    def _get_disallowed_fields_from_permissions(self, obj=None):
        return self._get_field_permissions().get_disallowed_fields(self.request, self, obj=obj)

    def _get_readonly_fields_from_permissions(self, obj=None):
        return self._get_field_permissions().get_readonly_fields(self.request, self, obj=obj)



from is_core.tests.factory.fields import delete_test_files


class DataGeneratorTestCase:

    factories = {}

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()
        delete_test_files()

    @classmethod
    def get_model_label(cls, model):
        return '%s.%s' % (model._meta.app_label, model._meta.object_name)

    def new_instance(self, model, only_build=False):
        model_label = self.get_model_label(model)
        if model_label in self.factories:
            if only_build:
                return self.factories.get(model_label).build()
            else:
                return self.factories.get(model_label)()

    def get_request_with_user(self, *args, **kwargs):
        request = super().get_request_with_user(*args, **kwargs)
        request.kwargs = {}
        return request



from django.conf import settings
from django.middleware import csrf

from pyston.serializer import serialize


def is_js_dev(req):
    return {'JS_DEV': settings.JS_DEV}


def initial_data(request):
    return {
        'initial_data': serialize({
            'net': {
                'csrf-token': csrf.get_token(request),
            },
        }),
    }



from django.contrib.auth.models import User

from germanium.test_cases.auth import UserProxy


class HelperTestCase:

    user_index = 1
    issue_index = 1

    def get_pk(self, resp):
        return self.deserialize(resp).get('id')

    def get_user_data(self, prefix=''):
        result = {
            'username': '%suser_%s' % (prefix, self.user_index),
            'email': '%suser_%s@test.cz' % (prefix, self.user_index),
            'password': 'super secret password'
        }
        self.user_index += 1
        return result

    def get_issue_data(self, prefix='', exclude=None):
        exclude = exclude or []
        result = {
            'name': 'Issue %s' % self.issue_index,
            'created_by': self.get_user_data(prefix),
            'leader': self.get_user_data(prefix)
        }
        self.issue_index += 1
        for field in exclude:
            del result[field]
        return result

    def get_user_obj(self, is_superuser=False):
        user_data = self.get_user_data()
        return User.objects._create_user(user_data.get('username'), user_data.get('email'),
                                         user_data.get('password'), is_staff=False, is_superuser=is_superuser)


class AsSuperuserTestCase:

    def get_user(self, is_superuser, is_staff=True):
        username = 'user'
        password = 'super secret password'
        email = 'user@test.cz'
        return UserProxy(username, password,
                         User.objects._create_user(username, email, password, is_staff=is_staff,
                                                   is_superuser=is_superuser))
